#!/usr/bin/perl

#
# Silicon Grail - preset conversion script
# 


use File::Find;
use Getopt::Std;

#######################################################################

$usage = "usage: $0 [-f -c chaliceroot -p presetdir] old_version new_version";

#######################################################################

# check command line

if ( ! getopts('fc:p:') )
{
	print STDERR "$usage\n";
	exit;
}


if ( $#ARGV != 1 )
{
	print STDERR "$usage\n";
	exit;
}

if ( $ARGV[0] !~ /[1-9]\.[1-9]/ || $ARGV[1] !~ /[1-9]\.[1-9]/ )
{
	print STDERR "$usage\n";
	exit;
}

if ( $ARGV[0] >= $ARGV[1] )
{
	print STDERR "convertPresets: src version must be less than dest\n";
	exit;
}

#######################################################################


# setup our variables

$oldVersion = $ARGV[0];
$newVersion = $ARGV[1];

$overWrite = 0;

if ( defined $opt_f )
{
	$overWrite = 1;
}

if ( defined $opt_c )
{
	if ( -e $opt_c )
	{
		$chalRoot = $opt_c;
	}
	else
	{
		print STDERR "convertPresets: directory does not exist: -c $opt_c\n";
		exit;
	}
}

if ( defined $opt_p )
{
	if ( -e $opt_p )
	{
		$presetDir = $opt_p;
	}
	else
	{
		print STDERR "convertPresets: directory does not exist: -p $opt_p\n";
		exit;
	}
}

if ( isWindowsNT() )
{
	$dirSep		= '\\';

	if ( ! defined $chalRoot )
	{
		$chalRoot	= $ENV{'CHALICEROOT'} 
		?  $ENV{'CHALICEROOT'} 
		: 'C:\\Program Files\\Silicon Grail\\Chalice' . $newVersion;
	}
	$converter  = $chalRoot . '\\support\\converters\\chaliceConvert.pl';

	$tmpFile1	= ($ENV{'TEMP'} 
						? $ENV{'TEMP'} . '\\' : 'C:\\') . $$ . '.grail';

	$tmpFile2	= ($ENV{'TEMP'} 
						? $ENV{'TEMP'} . '\\' : 'C:\\') . $$ . '.preset';

	if ( ! defined $presetDir )
	{
		$presetDir	= $ENV{'HOME'} 
						? $ENV{'HOME'} . '\\.chalice\\Presets' : 'C:\\';
	}

	$srcDir		= $presetDir . '\\' . $oldVersion;
	$targetDir	= $presetDir . '\\' . $newVersion;
}
else
{
	$dirSep		= '/';

	if ( ! defined $chalRoot )
	{
		$chalRoot = $ENV{'CHALICEROOT'} 
						? $ENV{'CHALICEROOT'} 
						: '/usr/grail/chalice' . $newVersion;
	}

	$converter  = $chalRoot . "/support/converters/chaliceConvert.pl";

	$tmpFile1	= ( $ENV{'TMPDIR'} ? $ENV{'TMPDIR'} . '/' : '/usr/tmp/' ) 
					. $$ . '.grail';

	$tmpFile2	= ( $ENV{'TMPDIR'} ? $ENV{'TMPDIR'} . '/' : '/usr/tmp/' ) 
					. $$ . '.preset';

	if ( ! defined $presetDir )
	{
		$presetDir	= $ENV{'HOME'} 
						? $ENV{'HOME'} . '/.chalice/Presets' 
						: '~/.chalice/Presets';
	}

	$srcDir		= $presetDir . '/' . $oldVersion;

	$targetDir	= $presetDir . '/' . $newVersion;
}

#######################################################################

# "main"

$ENV{'CHALICEROOT'} = $chalRoot;	# set it up for the converter

$newDir		= "";

find(\&wanted, $srcDir);

exit;

#######################################################################


# do the actual work

sub wanted {
	unless ( m/^.*\.preset$/ )
	{
		$newDir = $targetDir . $dirSep . $_;
		if ( ! -e $newDir )
		{
			print STDOUT "convertPresets: creating directory $newDir\n";
			if ( ! mkdir $newDir, 0755  )
			{
				print STDERR "convertPresets: unable to create directory $newDir\n";
			}
		}

		return;			# only do .preset
	}

	$newFile = $newDir . $dirSep . $_;

	if ( (-e $newFile) && (! $overWrite) )
	{
		print STDERR "convertPresets: skipping, file exists ($newFile)\n";
		return;
	}

	unless( open TMPFILE, ">$tmpFile1" )
	{
		print STDERR "convertPresets: unable to open temp file: $tmpFile1\n";
		exit;
	}
	unless( open SRCFILE, "<$File::Find::name" )
	{
		print STDERR "convertPresets: unable to open $oldVersion preset file: ";
		print STDERR "$File::Find::name\n";
		close TMPFILE;
		return;
	}

	print TMPFILE "# Chalice Version $oldVersion.1 by Silicon Grail\n";

	while ( $LINE = <SRCFILE> )
	{
		# some weirdness here, since regular grail files have type = <type>
		# and presets have type <type> - do the swap before calling the
		# converter which only knows about grail files not preset files
		$LINE =~ s/^type\s+(\w+)/type = $1/;
		print TMPFILE "$LINE";
	}

	close TMPFILE;
	close SRCFILE;

	unlink $tmpFile2;

	print STDOUT "convertPresets: converting $newFile\n";
	system("$converter $tmpFile1 $tmpFile2");

	unless ( open TMPFILE, "<$tmpFile2" )
	{
		print STDERR "convertPresets: unable to open tmp file: $tmpFile2\n";
		return;
	}

	unless ( open DESTFILE, ">$newFile" )
	{
		print STDERR "convertPresets: unable to open new preset file: ";
		print STDERR "$newFile\n";
		close TMPFILE;
		return;
	}

	print DESTFILE "#PSI_PRESET\n";

	# some weirdness here, since regular grail files have type = <type>
	# and presets have type <type> - do the swap
	while ( $LINE = <TMPFILE> )
	{
		if ( $LINE !~ m/^#/ )
		{
			$LINE =~ s/^type\s+=\s+(\w+)/type $1/;

			print DESTFILE "$LINE";
		}
	}

	unlink $tmpFile1;		# clean up the tmp files
	unlink $tmpFile2;
}

sub isWindowsNT
{
	return ( $^O eq 'NT' or $^O eq 'MSWin32' );
}

