#!/bin/csh -f

#
# Default/setup variables (generated by Install)
#
__default-setup-var__



#
# Sanity check, make sure script and executable are same version
#
set runMajorVer="`/bin/cat $CHALICEROOT/.VERSION`"
if ( $majorVer != $runMajorVer ) then
	echo "Sorry, you are trying to run the batch render for Chalice $runMajorVer with a"
	echo "script ($0) created to work with Chalice $majorVer."
	exit 1
endif


#
# Make sure we can run on this system
#
set osType=`/bin/uname -s | /bin/sed 's/64$//'`
set osVer=`/bin/uname -r`

if ( ($osType != "IRIX" && $osType != "Linux") || \
	 ($osType == "IRIX" && $osVer !~ 6.[2345]) ) then
	echo "The batch render for Chalice $majorVer doesn't run on $osType $osVer."
	echo "It only runs on IRIX 6.X and Linux."
	exit 1
endif

if ( "$osType" != "Linux" )  limit descriptors 2500


#
# Which directory to use for temp stuff
#
if ( ! $?CHALICE_TMPDIR ) then
	if ( $?TMPDIR ) then
		setenv CHALICE_TMPDIR  $TMPDIR
	else
		if ( $?TEMP ) then
			setenv CHALICE_TMPDIR  $TEMP
		else
			setenv CHALICE_TMPDIR  $default_tmpdir
		endif
	endif
endif


#
# Setup path for c compiler (needed for nodes that compile on the fly)
#
if ( $osType == "IRIX" ) then
	set gccLibBin=$CHALICEROOT/gnu/lib/gcc-lib/mips-sgi-irix6.2/2.7-irix6-970225
	set gccBin=$CHALICEROOT/gnu/bin
	set path=( $gccLibBin $gccBin $path )

	setenv TOOLROOT            "$CHALICEROOT/gnu/toolroot"
	setenv CHALICE_C_COMPILER  "$gccBin/gcc -O --specs=$gccLibBin/specs"
	setenv CHALICE_C_LINKER    "$TOOLROOT/usr/bin/ld -n32"
endif

if ( $osType == "Linux" ) then
	setenv CHALICE_C_COMPILER  "/usr/bin/gcc -O"
	setenv CHALICE_C_LINKER    "/usr/bin/ld"
endif


#
# Other variables added by user (generated by Install)
#
__user-added-vars__


#
# Actually run the program
#
$CHALICEROOT/bin/chal -T $$ $argv
set exitStatus=$status


#
# Cleanup after ourselves
#
set dir=$CHALICE_TMPDIR/Chalice.$$

if ( -e $dir ) then
	#
	# kill off dead files
	#
	foreach file ( `/usr/bin/find $dir -name \*.pids -print` )
		foreach pid ( `/usr/bin/cat $file` )
			(/bin/kill -9 $pid >& /dev/null)
		end
	end

	/bin/rm -rf $dir
endif


#
# Cleanup our shared memory
#
$CHALICEROOT/bin/chaliceShared -q $$


exit $exitStatus
