#!/usr/bin/perl
#
# Silicon Grail Chalice cleanup script.
#
# Script to cleanup ALL shared memory & semaphores that are hanging around
# that contain the magic value and the (optional) tag
#

require "getopts.pl";


if ( !&Getopts('hq') || $opt_h )
{
	@_ = split( /\//, $0 );
	$basename = pop(@_);

	print STDERR "usage: $basename [-q] [tag]\n";
	exit 0;
}

#
# Default/setup variables (generated by Install)
#
__default-setup-var__


#
# Sanity check, make sure script and executable are same version
#
open( VERSION, "< $ENV{'CHALICEROOT'}/.VERSION" ) || die( "ERROR: Couldn't read file `.VERSION': $!\n" );
$runMajorVer = <VERSION>;
chop($runMajorVer);
close WHICH;

if ( $majorVer ne $runMajorVer )
{
	die( "Sorry, you are trying to run the chaliceShared utility for Chalice  $runMajorVer with a\n" .
		 "script ($0) created to work with Chalice $majorVer\n" );
}


#
# Make sure we can run on this system
#
$osType = `/bin/uname -s`;
chop($osType);
$osType =~ s/64$//g;

$osVer = `/bin/uname -r`;
chop($osVer);
$osVer =~ s/^6\.[2-5]/6.X/ if ( $osType eq 'IRIX' );

if ( ( $osType ne 'IRIX' && $osType ne 'Linux') ||
	 ( $osType eq 'IRIX' && $osVer ne '6.X' ) )
{
	die( "The chaliceShared utility for Chalice $majorVer doesn't run on $osType $osVer.\n" .
		 "It only runs on IRIX 6.X and Linux." );
}


#
# On IRIX boxes we are expecting the ipcs output to look like this:
#
# >IPC status from /dev/kmem as of Thu Sep  9 12:28:13 1999
# >T         ID     KEY        MODE       OWNER    GROUP
# >Shared Memory:
# >m        223 0x00280267 --rw-r--r--     root      sys
#
# On Linux boxes we are expecting the ipcs output to look like this:
#
# >
# >------ Shared Memory Segments --------
# >key       shmid     owner     perms     bytes     nattch    status
# >0xfe113805 530049    you       777       256024    0
#
$quiet		= ($opt_q) ? "-q" : "";
$tag		= ($ARGV[0]) ? $ARGV[0] : 0;
$memchecker = "$ENV{'CHALICEROOT'}/bin/chaliceShm";

open( IPCSLINES, "/usr/bin/ipcs -m |" ) || die( "ERROR: Unable to get info from ipcs: $!\n" );

<IPCSLINES>;	# Skip the first three lines of junk
<IPCSLINES>;
<IPCSLINES>;


while( <IPCSLINES> )
{
	chop( $_ );

	if ( $osType eq "IRIX" )
	{
		$_ =~ s/^.*0x/0x/o;
	}

	($key, $junk ) = split(/\s+/, $_, 2);

	if ( $key =~ /^0x[0-9a-fA-F]{8}$/ )
	{
		system("$memchecker $quiet $key $tag");
	}
}
