/*
** To edit in vi, set ts=4
*/
#include  <stdio.h>
#include  <strings.h>
#include  <stdlib.h>
#include  <errno.h>
#include  <sys/types.h>

char  	*progName;

char *blurb[] = {
"usage:   fflip [-X] [-Y]\n\n",
"notes:   -X interchanges columns.\n",
"    -Y interchanges rows.\n\n",
""
};

void
Usage(void)
{
	fprintf( stderr, "usage: %s [-X] [-Y]\n", progName );
	exit( 1 );
}

/*
** Wed Dec 6 16:01:11 PST 1995 - William R. Bishop
**
** This is an example program that can be called from the chalice "Unix"
** node to flip or flop an image.  Note that image data is "raw" (no 
** compression, etc), and the image components are ordered RGBA.
**
** We let exit() handle free'ing the malloc'd arrays.
*/
main(
	int	argc, 
	char *argv[]
)
{
	int					 flip, flop;
	int					 xres, yres;
	int					 bytes, nComp, pixSize;
	u_char				*tmpBuf, *imgBuf;
	u_long				 nPix, i;
	register int 		 x, y;
	register u_char		*to, *from, *sl;

	/*
	** Strip off any leading pathname
	*/
	if( (progName = rindex( argv[0], '/' )) == NULL )
		progName = argv[0];
	else
		progName++; /* Step	over the '/' */

	/*
	** do a quick sanity check on the arguments
	*/
	if( argc < 2 )
	{
	   char	**foo = blurb;

	   while (**foo)
		 fprintf(stderr,*foo++);

	   exit( 1 );
	}

	flip = flop = 0;
	if( !strcmp( argv[1], "-X" ) )
		flop = 1;
	else if( !strcmp( argv[1], "-Y" ) )
		flip = 1;
	else
		Usage();

	/*
	** Get the size of the image; bytes is the # of bytes/image component.
	*/
	if( fread( &xres, sizeof(int), 1, stdin ) != 1 ||
		fread( &yres, sizeof(int), 1, stdin ) != 1 ||
		fread( &bytes,sizeof(int), 1, stdin ) != 1 ||
		fread( &nComp,sizeof(int), 1, stdin ) != 1 )
	{
		fprintf( stderr, "%s: Can't get image info!\n", progName );
		exit( 2 );
	}

	tmpBuf = imgBuf = NULL;

	nPix = xres * yres;
	pixSize = bytes * nComp;

	/*
	** Malloc our buffers
	*/
	if( (tmpBuf = malloc( xres*pixSize )) == NULL )
	{
		fprintf(stderr, "%s: Couldn't malloc tmp buffer.\n", progName );
		exit( 3 );
	}

	if( (imgBuf = malloc(nPix*pixSize)) == NULL )
	{
		fprintf(stderr, "%s: Memory allocation failed!\n", progName );
		exit( 3 );
	}

	/*
	** Read the image data
	*/
	if( fread( imgBuf, nPix*pixSize, 1, stdin ) != 1 )
	{
		fprintf(stderr, "%s: Couldn't read image because %s.\n", 
			progName, strerror( errno ) );
		exit( 4 );
	}

	/*
	** Do the processing
	*/
	if( flop )
	{
		register int end = xres/2;

		sl = imgBuf;
		for( y = 0; y < yres; y++, sl += (xres*pixSize) )
		{
			to = sl;
			from = sl + (xres-1)*pixSize;
			for( x = 0; x < end; x++ )
			{
				memcpy( tmpBuf, from, pixSize );
				memcpy( from, to, pixSize );
				memcpy( to, tmpBuf, pixSize );

				to += pixSize;
				from -= pixSize;
			}
		}
	}
	else
	{
		int	end = yres/2;
		int slSize = xres*pixSize;

		to = imgBuf;
		from = imgBuf + (yres-1)*slSize;
		for( y = 0; y < end; y++ )
		{
			memcpy( tmpBuf, from, slSize );
			memcpy( from, to, slSize );
			memcpy( to, tmpBuf, slSize );

			to += slSize;
			from -= slSize;
		}
	}

	/*
	** Write the processed image back out
	*/
	if( fwrite( imgBuf, nPix*pixSize, 1, stdout ) != 1 )
	{
		fprintf(stderr, "%s: Couldn't write image data because %s.\n", 
		progName, strerror( errno ) );
		exit( 5 );
	}
	fflush( stdout );

	exit( 0 );
}
