//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: Im.h
//
// COMMENTS:
//
/* $Id: Im.h,v 1.1.1.1 1999/10/06 21:12:43 build Exp $ */

#ifndef	IM_H
#define	IM_H

#ifndef NO_CHALICE
#define CHALICE
#endif
#if defined(CHALICE)
#include "ImChaliceFlags.h"
#endif

#if defined(sun) && !defined(SYSV)
#include <sys/limits.h>
#else
#include <limits.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>
#include <ErrorPP/ErrorPP.h>
#include <UT/UT_Standard.h>
#include <stdlib.h>
#include "ImError.h"
#include "ImDefs.h"
#include "ImTemplate.h"
#include "ImMatrix.h"

/* channel specifiers */

#define	IM_CHANNEL_MONO		0
#define	IM_CHANNEL_RED		0
#define	IM_CHANNEL_GREEN	1
#define	IM_CHANNEL_BLUE		2
#define	IM_CHANNEL_ALPHA	3
#define IM_CHANNEL_REAL		0
#define IM_CHANNEL_IMAG		1

#define IM_CHANNEL_CYAN		0
#define IM_CHANNEL_MAGENTA	1
#define IM_CHANNEL_YELLOW	2
#define IM_CHANNEL_BLACK	3

/* storage types */

#define IM_INTERLEAVED	0x0001
#define IM_PLANAR		0x0002
#define IM_BIT_PLANAR	0x0004
#define IM_COMPRESSED	0x0008

/* capabilities */

#define IM_CAP_NUM					30		// number of capabilites

#define IM_CAP_COLORMAP				0x00000001
#define IM_CAP_MULTIPLE				0x00000002
#define IM_CAP_MOTION				0x00000004

#define IM_CAP_TYPE_UNSIGNED		0x00000008
#define IM_CAP_TYPE_SIGNED			0x00000010
#define IM_CAP_TYPE_FLOAT			0x00000020
#define IM_CAP_TYPE_COMPLEX			0x00000040
#define IM_CAP_MULTIPLE_DTYPES		0x00000080

#define IM_CAP_SPACE_MONO			0x00000100
#define IM_CAP_SPACE_RGB			0x00000200
#define IM_CAP_SPACE_RGBA			0x00000400
#define IM_CAP_SPACE_CMYK			0x00000800
#define IM_CAP_SPACE_YUV_D1			0x00001000

#define IM_CAP_BPC_8				0x00002000
#define IM_CAP_BPC_16				0x00004000
#define IM_CAP_BPC_32				0x00008000
#define IM_CAP_BPC_OTHER			0x00010000

#define IM_CAP_COMPRESSION			0x00020000
#define IM_CAP_VARIABLE_COMPRESSION 0x00040000
#define IM_CAP_COMPRESS_ONLY		0x00080000

#define IM_CAP_NO_WRITE				0x00100000
#define IM_CAP_MUST_READ_LINEORDER	0x00200000
#define IM_CAP_MUST_WRITE_LINEORDER	0x00400000
#define IM_CAP_CAN_READ_PLANAR		0x00800000
#define IM_CAP_CAN_WRITE_PLANAR		0x01000000
#define IM_CAP_MUST_WRITE_PLANAR	0x02000000
#define IM_CAP_MUST_WRITE_REVERSE	0x04000000

#define IM_CAP_DEVICE				0x08000000
#define	IM_CAP_EXTENSION			0x10000000

#define	IM_CAP_SET_START			0x20000000
#define	IM_CAP_NO_FORMAT_SCAN		0x40000000

#define IM_SPACE_NUM				15	/* color spaces */
#define IM_SKIP_CHANNEL		0xFFFFFFFF	/* skip channel indicator */
#define IM_COUNT_UNKNOWN	0xFFFFFFFF	/* frame count error/can't compute */
#define IM_NOT_APPLICABLE			 0

#ifdef Complex
#undef Complex
#endif

class ImFormat;		// forward class declarations
class ImSpecs;

class Im {
public:
	//
	// The initial part of Im defines types.  If namespaces were available,
	// we would use them, but for now we define all the types here to
	// keep them safely in one name space.
	//
	typedef ::Int32			Int32;

	typedef ::Uint8			Uint8;
	typedef ::Uint16		Uint16;
	typedef ::Uint32		Uint32;

	typedef ::Float32		Float32;
	typedef ::Float64		Float64;

	typedef	Uint8			Uchar;
	typedef	Uint16			Ushort;
	typedef	Uint32			Uint;
	typedef	Uint32			Ulong;
	typedef Float32			Real;
	typedef	Int32			Int;
	typedef Float64			Fp;
	typedef	Uint32			Coord;		// image coordinate
	typedef	Int32			Bitfield;

	enum FileOp { READ, WRITE };

	//
	// The different color spaces that ImFiles supports.
	//
	enum ColorSpace	{ ImDefault=0, ImMono=1, ImInverse=2, ImComplex=3,
					  ImRGB=4, ImBGR=5, ImRGBA=6, ImARGB=7, ImABGR=8,
					  ImCMY=9, ImCMYK=10, ImYMC=11, ImYMCK=12, ImKCMY=13,
					  ImYUV_D1=14 };

	//
	// Color conversion mapping
	//
	enum Mapping	{ NoMapping, Default, Pseudo, Direct, Inverse,
					  CMYKtoRGB, RGBtoCMYK, YUVtoRGB, RGBtoYUV, RGBAtoYUV,
					  YUVtoCMYK, CMYKtoYUV, RGBtoMono, CMYKtoMono,
					  RGBAtoRGB, CMYKtoYMCK, CMYtoYMC, MonotoRGB };

	//
	// Data types that Im images can be
	//
	enum DataType	{ Unsigned, Signed, Float, Complex };


	//
	// This structure is used for converting a bitfield of capabilities
	// into a printable string.
	//
	struct CapTable
	{
		Bitfield	 cap;
		char		*capName;
	};

	//
	// This is a lookup table.  Given an index, it returns an integer value
	// corresponding to the index.
	//
	class ChannelMap : public ImArray<Uint> {
	public:
		ChannelMap( Uint size = 0, Uint bits = 0 );
		ChannelMap( const ChannelMap &a );
		virtual	~ChannelMap();

		Uint	 getSigBits( ) { return sigbits; }

	private:
		Uint	 sigbits;
	};

	//
	// This is how we keep track of an array of channel maps.
	// Typically, for a color indexed image this will have 3 elements.
	// One each for R, G, and B.
	//
	class ArrayChannel : public ImArray<ChannelMap *> {
	public:
		ArrayChannel( Uint size = 0 );
		ArrayChannel( Uint numChannels, Uint channelSize, Uint bits );
		ArrayChannel( const ArrayChannel &ac );
		virtual		~ArrayChannel();

	private:
		void		 acFree( );
	};

	// this	structure is used to call read() and write()
	class SubArea {
	public:
		 SubArea( const Im &image );
		 SubArea( );
		~SubArea( );
#ifdef WIN32
		Uint		bytesPerPixel() const;
		Uint		bytesPerChannel() const;
		Uint		bytesPerBitPlane() const;
		Uint		bytesPer() const;
#else
		Im::Uint	bytesPerPixel() const;
		Im::Uint	bytesPerChannel() const;
		Im::Uint	bytesPerBitPlane() const;
		Im::Uint	bytesPer() const;
#endif
		int			operator==( const SubArea &x ) const;

		void		print( FILE * ) const;	// useful for debugging

#ifdef WIN32
		ImArray<Uint>		origin; // origin of subarea in each dimension
		ImArray<Uint>		bounds; // bounds of subarea in each dimension
		ImArray<Uint>		whichChannels;	  // which channels are read/written
		ImArray<Uint>		sigBitsPerChannel;	// number of bits per channel
#else
		ImArray<Im::Uint>	origin; // origin of subarea in each dimension
		ImArray<Im::Uint>	bounds; // bounds of subarea in each dimension
		ImArray<Im::Uint>	whichChannels;	  // which channels are read/written
		ImArray<Im::Uint>	sigBitsPerChannel;	// number of bits per channel
#endif
		Mapping				mapped; // indirect values through map?
	};

public:
			 Im() { fileCapabilities = 0; }
	virtual	~Im();

	//
	// The specifications for a file are placed here.  This is not normally
	// called by user functions, except for adding multiple different images
	// to a file.
	//
	virtual ImBooln		 add( ImSpecs *d, ImDirection dir );
	virtual ImBooln		 remove();

	//
	// For files with multiple images, these functions allow access to
	// different images in the file.
	//
	// next		- Move on to the next image.
	// prev		- Go back to the previous image.
	// first	- Go to the first image.
	// last		- Go to the last image.
	// seek		- Go to a specific image. uses a string so that devices like
	//			  the Accom can access fields.
	// count	- Get the number of frames in a file/device.  returns
	//			  IM_COUNT_UNKNOWN if the image count cannot be figured out.
	//

	virtual ImBooln		 next();
	virtual ImBooln		 prev();
	virtual ImBooln		 first();
	virtual ImBooln		 last();
	virtual ImBooln		 seek( const char *imageNumber );
	virtual Uint		 count();
	virtual Uint		 start();			// Starting frame # (rtn 0 def)
	virtual Uint		 end();				// Ending frame (start+count-1)
	virtual ImBooln		 setStart( Uint );	// Set starting frame #

	//
	// The following retrieve details about the file data.
	//
	// bounds	- has dims() elements, gives the size of the image in pixels
	// bits		- has channels() elements, gives the total bits in a channel.
	// sigBits	- has channels() elements, gives significant bits in a channel.
	// maps		- returns the color lookup tables for mapped images.
	// matrix	- returns the pipeline transform matrix.
	//
	// dims		- the number of dimensions in an image.  Usually 2, could be
	//			  3 for volume data.
	// type		- the data type of the image data.
	// channels	- the number of channels in each pixel.  Usually 1, 3 or 4.
	// space	- the color space.  Usually RGB or RGBA.
	// attribs	- going away (I hope.)
	//
	// isPlanar		- data is stored a plane at a time.
	// isBitPlanar	- data is stored in 1 bit planes.
	// isInterleaved- data is stored pixel by pixel with all channels
	//				  contiguous in a pixel.
	// isCompressed	- data is compressed in the file.
	//
	virtual const ImArray<Uint>	&bounds( )	const;
	virtual const ImArray<Uint>	&bits( )	const;
	virtual const ImArray<Uint>	&sigBits( )	const;
	virtual const ArrayChannel	&maps( )	const;
	virtual		  ImMatrix		&matrix( );

	virtual Uint		 dims( )		const;
	virtual DataType	 type( )		const;
	virtual Uint 		 channels( )	const;
	virtual ColorSpace	 space( )		const;
	virtual Uint		 attribs( )		const;
	virtual ImBooln		 isPlanar( )	const;
	virtual ImBooln		 isBitPlanar( )		const;
	virtual ImBooln		 isInterleaved( )	const;
	virtual ImBooln		 isCompressed( )	const;

	//
	// These are the actual i/o requests.  The individual file formats
	// (or operators) are responsible for implementing these.
	//
	// read	-	places decoded file data into buffer.  buffer must contain
	//			at least area.bytesPer() bytes.  Channel data is always
	//			byte or short or word aligned. Eg. File data of 4 bits is
	//			rounded to 8 bits, 12 bits rounds to 16 bits, etc.
	//			Any subarea must be readable in any order.
	//
	// write -	puts buffer data into the file.  Just like read, the
	//			buffer data is assumed to be byte, short, or word aligned.
	//			bufArea describes the data as it is layed out in the buffer.
	//			outArea describes how the data should be written to the file.
	//			currently, only the whichChannels of the bufArea is important.
	//			Some (most) file formats need writes in a particular order.
	//			See the capabilites for details.
	//
	// cleanUp -Usually more important for writes than for reads, this function
	//			takes care of any final writes or other completion activities.
	//			The file is usually NOT closed here, however.  The destructor
	//			takes care of file closing.  You can safely assume that the
	//			destructor will be activated after a call to cleanUp.  CleanUp
	//			exists so that any errors during file final file manipulation
	//			can be caught.
	virtual	ImBooln		 read(  const SubArea &area, void *buffer ) = 0;
	virtual	ImBooln		 write( const SubArea &bufArea, void *buffer,
								const SubArea &outArea ) = 0;
	virtual ImBooln		 cleanUp();

	//
	// Get the file/class name of the file at the end of the imaging pipeline
	//
	virtual const char	*getFileName()  const = 0;
	virtual const char	*getClassName() const = 0;
	virtual const char	*getFormatName() const = 0;

	//
	// Find out if the file format can handle the passed in capability
	//
	virtual ImBooln		 fileSpec( const Bitfield cap )
		{ return ( fileCapabilities & cap ) == cap ? ImTrue : ImFalse; }

	//
	// These functions deal with imaging pipelines.
	//
	// getInput	   - returns the input Im of this Im or NULL.
	// deleteChild - recursivly deletes all children
	// deleteChild - deletes children up to, but not including, last.
	//
	virtual Im			*getInput() const;
	virtual ImBooln		 deleteChild();
	virtual ImBooln		 deleteChild( Im *last );

	//
	// caching methods
	//
	// setCache		   - sets the cache size
	// getNumCaches	   - recurses through the pipeline counting the
	//					 number of members needing a cache.
	// setCaching	   - turn on caching
	// recalcNumCaches - recalulate the number of current application caches in
	//					 order to provide client file objects with more cache
	//
	virtual ImBooln		 setCache( const Uint cacheSize );
	virtual Uint		 getNumCaches();
	virtual void		 setCaching( const ImBooln );

	virtual ImBooln		 recalcNumCaches( const Uint	 cacheSize,
										  const Uint	 imageBytes,
										  Uint			*numCaches );

	//
	// error gives everyone access to the error stack.  If any call returns
	// an error, the caller can retrieve the details from here
	//
	Error				 error;

	//
	// These are tables for printing user viewable string defining the
	// various capabilities and color spaces.
	//
	static const CapTable	 capTable[IM_CAP_NUM];
	static const char		*spaceTable[IM_SPACE_NUM];
	static const char		*typeTable[];

protected:
	Bitfield			 fileCapabilities;	// what i/o ops this image can do

private:
	ImList<ImSpecs>		 imageSpecs;		// list of specifications for
											// each image in the file

	ImMatrix			 xformMatrix;		// transformation matrix for
											// ImOp classes

};	/* end class Im */


//
// In general, there will one image specification per image in a file.
// (Movie files, where they are all the same, can have only one.)
//
// The various fields echo those in the Im class
//
class ImSpecs {
public:
	ImSpecs( const Im::Uint				 dims,
			 const ImArray<Im::Uint>	&bounds,
			 const Im::DataType			 t,
			 const Im::Uint				 channels,
			 const ImArray<Im::Uint>	&bits,
			 const ImArray<Im::Uint>	&bitsPer,
			 const Im::ColorSpace		 space,
			 const Im::ArrayChannel		&maps,
			 const Im::Uint				 attributes );
	~ImSpecs();

		  Im::Uint			 dims( )	const { return numDims; }
		  Im::DataType		 type( )	const { return channelType; }
		  Im::Uint			 channels( )const { return numChannels; }
		  Im::ColorSpace	 space( )	const { return colorSpace; }
		  Im::Uint			 attribs( )	const { return attributes; }
	const ImArray<Im::Uint>	&bounds( )	const { return dimBounds; }
	const ImArray<Im::Uint>	&bits( )	const { return bitsPerChannel; }
	const ImArray<Im::Uint>	&sigBits( )	const { return significantBits; }
	const Im::ArrayChannel	&maps( )	const { return channelMaps; }

	ImBooln	isPlanar()					const { return attributes & IM_PLANAR ?
													ImTrue : ImFalse; }
	ImBooln	isBitPlanar()				const { return attributes&IM_BIT_PLANAR?
													ImTrue : ImFalse; }
	ImBooln	isCompressed()				const { return attributes&IM_COMPRESSED?
													ImTrue : ImFalse; }
	ImBooln	isInterleaved()				const { return ImBooln(!isPlanar()); }

private:
	Im::Uint			numDims;		// number of dimensions
	ImArray<Im::Uint>	dimBounds;		// bounds in each dimension
	Im::DataType		channelType;	// data type
	Im::Uint			numChannels;	// number of channels
	ImArray<Im::Uint>	bitsPerChannel;	// number of bits per channel
	ImArray<Im::Uint>	significantBits;// number of significant channel
	Im::ColorSpace		colorSpace;		// Color space
	Im::ArrayChannel	channelMaps;	// look-up tables for channels
	Im::Uint			attributes;		// image attributes
}; /* end class ImSpecs */

#endif /* IM_H */
