//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImArray.h
//
// COMMENTS:
//	
/* $Id: ImArray.h,v 1.2 1999/10/29 03:22:12 kimball Exp $ */

#include <ErrorPP/ErrorPP.h>
#include <UT/UT_Standard.h>

#ifndef IM_ARRAY_H
#define IM_ARRAY_H

template <class T>
class ImArray {
public:
	inline ImArray( Uint32 size = 0 )
		{
			if ( ISNULL( v = new T [size] ) ) {
				error.V( E_NONE, E_MALLOC );
				len = 0;
			}
			else
				len = size;
		}
	inline ImArray( const ImArray<T> &a )			{ copy( a ); }
	virtual			~ImArray( void )						{ free(); }

	inline ImArray<T>		&operator=( const ImArray<T> &a )
		{
			if ( this != &a ) {
				free();
				copy( a );
			}
			return *this;
		}

	inline ImArray<T>		&operator=( const T a )
		{
			for ( Uint32 i=0; i<len; i++ )
				v[i] = a;
			return *this;
		}
	inline int				 operator==( const ImArray<T> &a ) const
		{
			if ( a.len == len ) {
				for ( Uint32 i=0; i<len; i++ )
					if ( a[i] != v[i] ) return 0;
				return 1;
			}

			return 0;
		}
	inline int				 operator==( const T a ) const
		{
			for ( Uint32 i=0; i<len; i++ )
				if ( v[i] != a ) return 0;
			return 1;
		}
	inline T				&operator[]( const Uint32 i )
		{ return index(i); }
	inline T				 operator[]( const Uint32 i ) const
		{ return index(i); }

	inline T				*raw( )
		{ return v; }

	inline Uint32			 size( ) const
		{return len;}
	inline void			 size( Uint32 nlen )
		{
			UT_ASSERT( nlen > 0 );
			if ( nlen != len ) {
				SAFE_ARRDELETE( v );
				if ( ISNULL( v = new T [nlen] ) ) {
					error.V( E_NONE, E_MALLOC );
					len = 0;
				}
				else
					len = nlen;
			}
		}

	inline void			 print( FILE *fp ) const {}
	inline void			 sort( )
		{
			qsort((void*)v,len,sizeof(T),(int (*)(const void *, const void *))compare);
		}
	inline void			 unique( )
		{
			Uint32 newlen = (Uint32)len;

			for ( Uint32 i = 0; i < newlen - 1; i++ )	{
				Uint32 j = i + 1;
				while ( j < newlen && compare( &v[i], &v[j] ) == 0 )
					j++;
				if ( j > i + 1 )
				{
					if ( j == newlen )
						newlen = i + 1;
					else
					{
						memmove( &v[i+1], &v[j], newlen - j );
						newlen -= j - i + 1;
					}
				}
			}

			if ( newlen != (Uint32)len ) {
				T *nv = new T [len = (int)newlen];
				for ( Uint32 i = 0; i < len; i++ )
					nv[i] = v[i];
				SAFE_ARRDELETE( v );
				v = nv;
			}
		}
	inline void			 ident( )
		{
			for ( Uint32 i = 0; i < len; i++ )
				v[i] = i;
		}

private:
	inline void			 copy( const ImArray<T> &vl )
		{
			v = new T [len = vl.len];
			for ( Uint32 i=0; i<len; i++ )
				v[i] = vl.v[i];
		}
	inline void			 free( void )
		{	SAFE_ARRDELETE( v ); }
	inline T			&index( const Uint32 i ) const
		{ UT_ASSERT( i < len ); return v[i]; }
	// for sorting
	static inline int		 compare( T *a, T *b )
		{
			return *a - *b;
		}

	Error			 error;
	Uint32			 len;
	T				*v;
};	/* class ImArray */

#endif
