//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImCdi.h
//
// COMMENTS:
//
/* $Id: ImCdi.h,v 1.1.1.1 1999/10/06 21:12:44 build Exp $ */

#ifndef IMCDI_H
#define IMCDI_H

#include "ImFile.h"
#include <UT/UT_Endian.h>

struct CDIHeader
{
	Uint32		magic[2];			// magic identifier
	Uint32		version;			// version of the Cdi format
	Uint32		width, height;		// image size in pixels
	Uint32		channels;			// image channels 1-N
	Uint32		bpc;				// bytes per channel, chalice supports:
									// 1 (unsigned byte), 2 (unsigned short),
									// and 4 (float)
	char		processor;			// created on MOTOROLA/INTEL
};

class ImCdi : public ImFile {
public:
	//
	// read constructor
	//
	ImCdi(const char			*fname,
		  FILE					*file,
		  const Uint32			 perm
		  );

	//
	// write constructor
	//
	ImCdi(const char			*name,
		  FILE					*file,
		  const Uint32			 perm,
		  const Uint			 numDims,			// number of dimensions
		  const ImArray<Uint>	&dimBounds,			// bounds in each dimen
		  const Uint			 numChannels,		// number of channels
		  const ImArray<Uint>	&bitsPerChannel,	// num of bits per channel
		  const ImArray<Uint>	&significantBits,	// num of signif. channel
		  const DataType		 channelType,		// data type
		  const ColorSpace		 colorSpace,		// Color space
		  const ArrayChannel	&channelMaps,		// channel lookup tables
		  const Uint			 storageType		// compressed?
		  );

	virtual				~ImCdi();

	//
	// read, write and cleanup for file i/o
	//
	virtual	ImBooln		 read( const SubArea &area, void *buffer );
	virtual	ImBooln		 write( const SubArea &bufArea, void *buffer,
								const SubArea &outArea );
	virtual ImBooln		 cleanUp();

	//
	// 'code' and english name
	//
	virtual const char	*getClassName() const;
	virtual const char	*getFormatName() const;

	static const char	*className;
	static const char	*formatName;
	static void  swapHeader( CDIHeader *head );

private:
	//
	// ImCdi cannot be copied or assigned.
	//
				 ImCdi(const ImCdi &);			//lint !e1704
	ImCdi		&operator=(const ImCdi &);		//lint !e1704

	//
	// Seek to the given x, y location in the file
	//
	ImBooln		 iseek( Uint x, Uint y );

	//
	// The line we are currently writing to.  Make sure we do not write
	// out of order.
	//
	Uint		 myWriteLine;
};	/* end class ImCdi */

// force loading of ImCdi.o during link phase
#ifndef IM_DSO
static const char *forceCdiLoad() { return ImCdi::className; }
#endif

class ImCdiMaker : public ImFileMaker {
public:
	ImCdiMaker( ImExemplar e );
	virtual				~ImCdiMaker();

private:
	virtual Im			*openIt( const char		*name,
								 FILE			*fp,
								 Uint32			 perm,
								 ImBooln		 noSearch );

	virtual Im			*openIt( const char					*name,
								 FILE						*file,
								 Uint32						 permission,
								 Im::Uint					 numDims,
								 const ImArray<Im::Uint>	&bounds,
								 Im::Uint					 numChannels,
								 const ImArray<Im::Uint>	&bits,
								 const ImArray<Im::Uint>	&sigBits,
								 Im::DataType				 dataType,
								 Im::ColorSpace				 space,
								 const Im::ArrayChannel		&maps,
								 Im::Uint					 attributes );

	virtual ImBooln		 getFormat(const char			*name,
								   FILE					*file,
								   const struct stat	*statinfo,
								   Im::Uint				*width,
								   Im::Uint				*height,
								   Im::Uint				*channels,
								   Im::Uint				*bits,
								   Im::DataType			*type,
								   Im::Bitfield			*caps);

	virtual Im::Uint	 estimateSize( const ImArray<Im::Uint>	&bounds,
									   const Im::Uint			 numChannels,
									   const ImArray<Im::Uint>	&bits
									   ) const;

#if 0					// CDI uses the base class version of isExtension
	virtual ImBooln		 isExtension( const char *ext );
#endif

	ImBooln				 getHeader( FILE *fp, CDIHeader & );
};	/* end class ImCdiMaker */

#endif /* IMCDI_H */
