//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImCopy.C
//
// COMMENTS:
//	
#include <UT/UT_Standard.h>
#include "Im.h"
#include "ImCopy.h"


////////////////////////////////////////


ImCopy::ImCopy()
{
}

////////////////////////////////////////


ImCopy::~ImCopy()
{
}

////////////////////////////////////////


#ifdef NORMAL_TEMPLATES
#else
template <class PXL>
ImReorderIn<PXL>::ImReorderIn(Im::Uint			 srcPixelsPerLine,
							  Im::Uint			 srcPixelsOffset,
							  Im::Uint			 srcChannelsPerPixel,
							  Im::Uint			 dstPixelsPerLine,
							  Im::Uint			 dstPixelsOffset,
							  Im::Uint			 copyPixelsPerLine,
							  ImArray<Im::Int>	&srcMoves) :
		mySrcMoves(srcMoves)
{
	mySrcPixelsPerLine		= srcPixelsPerLine;
	mySrcPixelsOffset		= srcPixelsOffset;
	mySrcChannelsPerPixel	= srcChannelsPerPixel;
	myDstPixelsPerLine		= dstPixelsPerLine;
	myDstPixelsOffset		= dstPixelsOffset;
	myCopyPixelsPerLine		= copyPixelsPerLine;
}

template <class PXL>
void *
ImReorderIn<PXL>::copy(const void	*src,
					   void			*dst,
					   Im::Uint		 lines)
{
	PXL			*srcp = (PXL *)src;
	PXL			*dstp = (PXL *)dst;
	Im::Int		*movp = mySrcMoves.raw();
	Im::Uint	 pxls = mySrcMoves.size() - 1;
	Im::Uint	 sstep	= mySrcPixelsPerLine * mySrcChannelsPerPixel;
	Im::Uint	 dstep	= myDstPixelsPerLine * pxls;

	srcp += mySrcPixelsOffset * mySrcChannelsPerPixel;	//goto the 1st src pixel
	dstp += myDstPixelsOffset * pxls;					//goto the 1st dst pixel

	while ( lines > 0 ) {
		PXL *s = srcp;
		PXL *d = dstp;
		PXL *e = dstp + myDstPixelsPerLine * pxls;

		while ( d < e ) {							// do one line
			Im::Int	*m = movp;
			for ( Im::Uint i = pxls; i-- > 0; ) {	// for all the dst pels
				s += *m++;							// move to the first pel
				*d++ = *s;							// copy the pel
			}
			s += *m;								// move to start of next pxl
		}

		srcp += sstep;								// move to the next line
		dstp += dstep;
		lines--;
	}

	return (void *)dstp;
}

template <class PXL>
ImCopy *
ImReorderIn<PXL>::create(Im::Uint			 srcPixelsPerLine,
						 Im::Uint			 srcPixelsOffset,
						 Im::Uint			 srcChannelsPerPixel,
						 Im::Uint			 dstPixelsPerLine,
						 Im::Uint			 dstPixelsOffset,
						 Im::Uint			 copyPixelsPerLine,
						 ImArray<Im::Int>	&srcMoves)
{
	Im::Uint	pixels = srcMoves.size() - 1;

	switch ( pixels ) {
	case 1:
		return new ImReorderIn1<PXL>(srcPixelsPerLine,
									 srcPixelsOffset,
									 srcChannelsPerPixel,
									 dstPixelsPerLine,
									 dstPixelsOffset,
									 copyPixelsPerLine,
									 srcMoves);
	case 3:
		return new ImReorderIn3<PXL>(srcPixelsPerLine,
									 srcPixelsOffset,
									 srcChannelsPerPixel,
									 dstPixelsPerLine,
									 dstPixelsOffset,
									 copyPixelsPerLine,
									 srcMoves);

	case 4:
		return new ImReorderIn4<PXL>(srcPixelsPerLine,
									 srcPixelsOffset,
									 srcChannelsPerPixel,
									 dstPixelsPerLine,
									 dstPixelsOffset,
									 copyPixelsPerLine,
									 srcMoves);

	default:
		return new ImReorderIn<PXL>(srcPixelsPerLine,
									srcPixelsOffset,
									srcChannelsPerPixel,
									dstPixelsPerLine,
									dstPixelsOffset,
									copyPixelsPerLine,
									srcMoves);
	}
}

template <class PXL>
void *
ImReorderIn1<PXL>::copy(const void	*src,
						void		*dst,
						Im::Uint	 lines)
{
	PXL			*srcp = (PXL *)src;
	PXL			*dstp = (PXL *)dst;
	Im::Int		*movp = mySrcMoves.raw();
	Im::Uint	 sstep	= mySrcPixelsPerLine * mySrcChannelsPerPixel;
	Im::Uint	 dstep	= myDstPixelsPerLine;

	srcp += mySrcPixelsOffset * mySrcChannelsPerPixel;	//goto the 1st src pixel
	dstp += myDstPixelsOffset;							//goto the 1st dst pixel

	while ( lines > 0 ) {
		PXL *s = srcp;
		PXL *d = dstp;
		PXL *e = dstp + myDstPixelsPerLine;

		while ( d < e ) {			// copy all the pels in one line
			Im::Int	*m = movp;
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m;				// move to the start of the next pixel
		}

		srcp += sstep;				// move to the next line
		dstp += dstep;
		lines--;
	}

	return (void *)dstp;
}

template <class PXL>
void *
ImReorderIn3<PXL>::copy(const void	*src,
						void		*dst,
						Im::Uint	 lines)
{
	PXL			*srcp = (PXL *)src;
	PXL			*dstp = (PXL *)dst;
	Im::Int		*movp = mySrcMoves.raw();
	Im::Uint	 sstep	= mySrcPixelsPerLine * mySrcChannelsPerPixel;
	Im::Uint	 dstep	= myDstPixelsPerLine * 3;

	srcp += mySrcPixelsOffset * mySrcChannelsPerPixel;	//goto the 1st src pixel
	dstp += myDstPixelsOffset * 3;						//goto the 1st dst pixel

	while ( lines > 0 ) {
		PXL *s = srcp;
		PXL *d = dstp;
		PXL *e = dstp + myDstPixelsPerLine * 3;

		while ( d < e ) {			// copy all the pels in one line
			Im::Int	*m = movp;
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m;				// move to the start of the next pixel
		}

		srcp += sstep;				// move to the next line
		dstp += dstep;
		lines--;
	}

	return (void *)dstp;
}

template <class PXL>
void *
ImReorderIn4<PXL>::copy(const void	*src,
						void		*dst,
						Im::Uint	 lines)
{
	PXL			*srcp = (PXL *)src;
	PXL			*dstp = (PXL *)dst;
	Im::Int		*movp = mySrcMoves.raw();
	Im::Uint	 sstep	= mySrcPixelsPerLine * mySrcChannelsPerPixel;
	Im::Uint	 dstep	= myDstPixelsPerLine * 4;

	srcp += mySrcPixelsOffset * mySrcChannelsPerPixel;	//goto the 1st src pixel
	dstp += myDstPixelsOffset * 4;						//goto the 1st dst pixel

	while ( lines > 0 ) {
		PXL *s = srcp;
		PXL *d = dstp;
		PXL *e = dstp + myDstPixelsPerLine * 4;

		while ( d < e ) {			// copy all the pels in one line
			Im::Int	*m = movp;
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m++;				// move to the pel to copy
			*d++ = *s;				// copy the pel
			s += *m;				// move to the start of the next pixel
		}

		srcp += sstep;				// move to the next line
		dstp += dstep;
		lines--;
	}

	return (void *)dstp;
}

#endif
Im::Uchar *
ImCopy::fullLines(const Im::Uchar	*src,
				  Im::Uchar			*dst,
				  Im::Uint			 bytesPerLine,
				  Im::Uint			 lines)
{
	Im::Uint total = bytesPerLine * lines;

	return (Im::Uchar *)memcpy(dst, src, total) + total;
}

Im::Uchar *
ImCopy::partialLines(const Im::Uchar	*src,
					 Im::Uchar			*dst,
					 Im::Uint			 srcBytesPerLine,
					 Im::Uint			 dstBytesPerLine,
					 Im::Uint			 copyBytesPerLine,
					 Im::Uint			 lines)
{
	while ( lines > 0 )
	{
		memcpy(dst, src, copyBytesPerLine);
		src += srcBytesPerLine;
		dst += dstBytesPerLine;
		lines--;
	}

	return dst;
}
