//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImError.h
//
// COMMENTS:
//	
#ifndef IM_ERROR_H
#define IM_ERROR_H

enum ImErrorTypes {
	E_TYPE_BITS,
	E_TYPE_BOUNDS,
	E_TYPE_BYTES,
	E_TYPE_CACHE,
	E_TYPE_CHANNELS,
	E_TYPE_CHANNEL_TYPE,
	E_TYPE_CINEON_CONVERSION,
	E_TYPE_CMAP,
	E_TYPE_COLORSPACE,
	E_TYPE_DIMENSIONS,
	E_TYPE_DITHER,
	E_TYPE_FORMAT,
	E_TYPE_INVALID_VALUE,
	E_TYPE_JPEG_LIB,
	E_TYPE_MISC,
	E_TYPE_MMGR,
	E_TYPE_MODE,
	E_TYPE_NOT_SUPPORTED,
	E_TYPE_OPEN_DEVICE,
	E_TYPE_PERMISSION,
	E_TYPE_PLANAR,
	E_TYPE_READ_DEVICE,
	E_TYPE_READ_STREAM,
	E_TYPE_ROW_SEQUENCE,
	E_TYPE_STAMP,
	E_TYPE_TAG,
	E_TYPE_TIFF_LIB,
	E_TYPE_TYPE,
	E_TYPE_WRITE_DEVICE
};


#define E_NOT_SUPPORTED( field, value, fileType ) \
		E_BASE_IM + E_TYPE_NOT_SUPPORTED, \
		"%s of `%d' not supported for %s files!", field, value, fileType

#define E_INVALID_VALUE( field, value, fileType ) \
		E_BASE_IM + E_TYPE_INVALID_VALUE, \
		"Invalid %s of `%d' for %s file!", field, value, fileType

#define E_MALLOC_FIELD( field, fileType ) E_BASE_IM + E_TYPE_MALLOC, \
		"Cannot allocate memory for %s for %s file!", field, fileType

#define E_ONLY_2_D E_BASE_IM + E_TYPE_NOT_SUPPORTED, \
		"Can only resample 2D images"

#define E_CHANNELS( Channels, fileType ) E_BASE_IM + E_TYPE_CHANNELS, \
        "Number of channels must be %s for %s files!", Channels, fileType

#define E_CHANNELS_SEQUENCE( ord, fileType ) E_BASE_IM + E_TYPE_CHANNELS, \
        "Channels must be written in %s order for %s files!", ord, fileType

#define E_CHANNEL_TYPE( Type, fileType ) E_BASE_IM + E_TYPE_CHANNEL_TYPE, \
        "Channel type must be %s for %s files!", Type, fileType

#define E_BITS( bits, fileType ) E_BASE_IM + E_TYPE_BITS, \
        "Bits per channel must be %s for %s files!", bits, fileType

#define E_DIMS( numDims, fileType ) E_BASE_IM + E_TYPE_DIMENSIONS, \
        "Number of dimensions must be %s for %s files!", numDims, fileType

#define E_BYTES( bytes, fileType ) E_BASE_IM + E_TYPE_BYTES, \
        "Bytes per channel must be %s for %s files!", bytes, fileType

#define E_BYTES_UNIFORM( fileType ) E_BASE_IM + E_TYPE_BYTES, \
        "Bytes per channel must be same between channels in %s file!", \
		fileType

#define E_BITS_UNIFORM( fileType ) E_BASE_IM + E_TYPE_BYTES, \
        "Bits per channel must be same between channels in %s file!", \
		fileType

#define E_COLORSPACE( colorspace, fileType ) E_BASE_IM + E_TYPE_COLORSPACE, \
        "Colorspace must be %s for %s files!", colorspace, fileType

#define E_ROW_SEQUENCE( fileType ) E_BASE_IM + E_TYPE_ROW_SEQUENCE, \
        "Must write out rows in sequence for %s files!", fileType

#define E_STAMP( errno ) E_BASE_IM + E_TYPE_STAMP, \
		"Invalid Stamp File -- Error %d!", errno

#define E_READ_ITEM( item, fileType ) E_BASE_IM + E_TYPE_READ_FILE, \
        "Could not read %s from %s file because `%s'!", \
		item, fileType, strerror( errno )

#define E_WRITE_ITEM( item, fileType ) E_BASE_IM + E_TYPE_WRITE_FILE, \
        "Could not write out %s to %s file because `%s'!", \
		item, fileType, strerror( errno )

#define E_CROP_BOUNDS E_BASE_IM + E_TYPE_BOUNDS, \
		"Crop bounds outside of image bounds!"

#define E_UPDATE_COMPRESSED( fileType ) E_BASE_IM + E_TYPE_MISC, \
		"Cannot modify compressed %s file. Use read permission only!" \
		fileType

#define E_FILE_PERM( perm, action, name ) E_BASE_IM + E_TYPE_PERMISSION, \
		"Bad permission, %d, for %s file %s!", perm, action, name

#define E_CMAP_SIZES( fileType ) E_BASE_IM + E_TYPE_CMAP, \
		"Colormap sizes for each channel of a %s file must be equal!", \
		fileType

#define E_IM_FORMAT( fmt ) E_BASE_IM + E_TYPE_FORMAT, \
		"Can't find format `%s'", \
		fmt

#define E_IM_FORMAT_FILE( fmt, name ) E_BASE_IM + E_TYPE_FORMAT, \
		"Can't find format `%s' or open `%s'", \
		fmt

#define E_CACHE( imageBytes, cacheSize ) E_BASE_IM + E_TYPE_CACHE, \
		"Cache, %d, inefficient size for input image. Increase cache \
		to at least %d bytes to increase efficiency!", \
		 cacheSize, imageBytes

#endif  /* IM_ERROR_H */
