//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImFile.h
//
// COMMENTS:
//
/* $Id: ImFile.h,v 1.1.1.1 1999/10/06 21:12:45 build Exp $ */

#ifndef IM_FILE_H
#define IM_FILE_H

#include <string.h>
#include <math.h>
#include "Im.h"
#include "ImOpt.h"
#include "ImFileMaker.h"

/*
 * defines for permissions argument to ImFile constructors
 */
#define IM_FILE_READ	0x0001
#define IM_FILE_WRITE	0x0002
#define IM_FILE_CREATE	0x0004

extern "C" {
	struct stat;
}

#ifdef IM_DSO
class UT_FileName;
#endif

class ImFile : public Im {
public:

	ImFile();

	//
	// constructor used to read/write an existing file
	//
	ImFile( const char			*name,
			FILE				*file,
			const Uint32	 perm );

	//
	// constructor used to create a new file or to open
	// a new one for raw files
	//
	ImFile(
		const char				*name,
			  FILE				*file,
		const Uint32			 perm,
		const Uint				 numDims,		// number of dimensions
		const ImArray<Uint>		&dimBounds,		// bounds in each dimension
		const Uint				 numChannels,	// number of channels
		const ImArray<Uint>		&bitsPerChannel,// number of bits per channel
		const ImArray<Uint>		&significantBits,// number of significant bits
		const DataType			 channelType,	// data type
		const ColorSpace		 colorSpace,	// Color space
		const ArrayChannel		&channelMaps,	// channel lookup tables
		const Uint				 attributes		// image attributes
	);

	virtual ~ImFile();

	virtual const char	*getFileName() const;
	virtual void		 setCaching( const ImBooln );

protected:
	//
	// Confirm the colorspace and capabilities or colorspace and channels match
	//
	ImBooln		 validColorSpace( const ColorSpace colorSpace,
								  const Bitfield cap );

	ImBooln		 validColorSpace( const Im::ColorSpace colorSpace,
								  const Im::Uint numChannels );

	void		 nameSet( const char * const name_ );

	FILE		*fp;			// The open file
	char		*name;			// The file name
	ImBooln		 ourUseCache;	// Caching is on/off
	ImBooln		 ourWritingFile;// We are writting to the file

private:
	//
	// ImFile cannot be copied or assigned.
	//
				 ImFile(const ImFile &);		//lint !e1704
	ImFile		&operator=(const ImFile &);		//lint !e1704

	//
	// convert the perm into a FILE type string
	//
	char		*permStr( const char * const name_, const Uint32 perm );

	//
	// Handle the actual opening of the file
	//
	ImBooln		 open( const char * const	 name,
					   FILE					*file,
					   const Uint32			 perm );

	//
	// Make sure we can actually create this file.
	//
	int			 createCheck(
		const Uint32		 perm,
		const Uint			 numDims,			// number of dimensions
		const ImArray<Uint>	&dimBounds,			// bounds in each dimension
		const Uint			 numChannels,		// number of channels
		const ImArray<Uint>	&bitsPerChannel,	// number of bits per channel
		const ImArray<Uint>	&significantBits,	// number of significant channel
		const DataType		 channelType,		// data type
		const ColorSpace	 colorSpace,		// Color space
		const ArrayChannel	&channelMaps,		// channel lookup tables
		const Uint			 attributes,		// image attributes
			  Uint			&newPerm			// new permissions for open
	);
};

#endif /* IM_FILE_H */
