//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImList.C
//
// COMMENTS:
//	

#include "ImList.h"
#include <UT/UT_Standard.h>

template <class T>
ImList<T>::ImList()
{
	head = tail = current = NULL;
	count = 0;
}

template <class T>
ImList<T>::~ImList()
{

	current = head;

	while ( current ) {
		ImListItem<T> *nxt = current->next;
		SAFE_DELETE( current );
		current = nxt;
		--count;
	}
	UT_ASSERT( count == 0 );
}

// this remove does NOT delete space
template <class T>
void
ImList<T>::remove( ImListItem<T> *which )
{
	if ( which->prev )
		which->prev->next = which->next;
	if ( which->next )
		which->next->prev = which->prev;

	if ( head == which )
			head = which->next;
	if ( tail == which )
			tail = which->prev;
}

// this remove deletes list item
template <class T>
ImBooln
ImList<T>::remove()
{

	if	( current )	{
		ImListItem<T> *n = current->next;
		remove( current );
		SAFE_DELETE( current );
		current =	( n	? n	: tail );
	}
	else
		return ImFalse;

	--count;
	return ImTrue;
}

template <class T>
void
ImList<T>::add(ImListItem<T>	*node,
			   ImListItem<T>	*where,
			   ImDirection		 direction )
{
	if	( direction	== Before )	{
		node->next = where;
		node->prev = where ? where->prev : 0;
		if ( where )
			where->prev = node;
		if	( node->prev )	node->prev->next = node;
	} else	{
		node->prev = where;
		node->next = where ? where->next : 0;
		if ( where )
			where->next = node;
		if	( node->next )	node->next->prev = node;
	}

	if ( ! node->next )	tail = node;
	if ( ! node->prev )	head = node;
}

template <class T>
ImBooln
ImList<T>::add( T *t, ImDirection direction )
{

	/*
	 * note: the data item, t, should be allocated by the calling
	 *		 routine. However, it is deleted by the ImListItem
	 *		 destructor so the user should not free it.
	 */

	ImListItem<T> *node;

	if ( current ) {

		if	( direction	== Before )	{

			node = new ImListItem<T>( t, current, current->prev );
			current->prev = node;
			if	( node->prev )	node->prev->next = node;
		} else	{

			node = new ImListItem<T>( t, current->next, current );
			current->next = node;
			if	( node->next )	node->next->prev = node;
		}

	}
	else
		node = new ImListItem<T>( t, 0, 0 );

	if ( ! node->next )	tail = node;
	if ( ! node->prev )	head = node;

	current	= node;

	++count;
	return ImTrue;
}

template <class T>
T *
ImList<T>::next( ) {

	if ( current ) {

		current = current->next;
		if	( current )
			return	current->data;
		else
			return	0;
	}
	else
		return	0;
}

template <class T>
T *
ImList<T>::prev( )
{

	if	( current )	{
		current = current->prev;
		if ( current )
			return current->data;
		else
			return 0;
	} else
		return 0;
}


template <class T>
T *
ImList<T>::first( )
{
	current = head;
	if	( current )	{
		return current->data;
	} else
		return 0;
}

template <class T>
T *
ImList<T>::last( )
{
	current = tail;
	if	( current )	{
		return current->data;
	} else
		return 0;
}

template <class T>
void
ImList<T>::moveFirst( )
{
	if ( current != head ) {
		remove( current );
		add( current, head, Before );
	}
}

template <class T>
void
ImList<T>::moveLast( )
{
	if ( current != tail ) {
		remove( current );
		add( current, tail, After );
	}
}

template <class T>
const T *
ImList<T>::data( ) const
{
	if	( current )	{
		return current->data;
	} else
		return 0;
}


template <class T>
ImListItem<T>::~ImListItem()
{
	SAFE_DELETE( data );
}
