//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImOpt.h
//
// COMMENTS:
//	
/* $Id: ImOpt.h,v 1.1.1.1 1999/10/06 21:12:46 build Exp $ */

#ifndef	IM_OPT_H
#define	IM_OPT_H

#include <stddef.h>

//
// These flags apply to all option types
//
const Uint32 ImOptIn		= 0x0001;	//option applies to reading
const Uint32 ImOptOut		= 0x0002;	//option applies to writing
const Uint32 ImOptIO		= 0x0003;	//option applies to reading and writing

//
// These flags apply to integer and floating point options only
//
const Uint32 ImOptHardMin	= 0x0004;
const Uint32 ImOptHardMax	= 0x0008;
const Uint32 ImOptHardMM	= 0x000C;

class ImFileMaker;
class ImFileOptStow;

//
// For menus and Option names, we define two 'names' the english (user viewable)
// name and the tag.  The tag can be used to save in files, and for the internal
// value.  We do this so that you can change the english name without having to
// change any other code.
//
struct ImOptName {
	inline ImOptName(const char *e, const char *t);
	inline ImOptName(const ImOptName &a);
	const char	*myEnglish;		// What the user sees
	const char	*myTag;			// the 'real' value for saving in files, etc.
};

class ImOpt
{
public:
	ImOpt(const ImOptName	&name,
		  Uint32			 dValue,
		  Uint32			 flags);

	ImOpt(const ImOptName	&name,
		  Uint32			 count,
		  int				 dValue,
		  int				 min,
		  int				 max,
		  Uint32			 flags);

	ImOpt(const ImOptName	&name,
		  Uint32			 count,
		  int				*dValue,
		  int				 min,
		  int				 max,
		  Uint32			 flags);

	ImOpt(const ImOptName	&name,
		  Uint32			 count,
		  float				*dValue,
		  float				 min,
		  float				 max,
		  Uint32			 flags);

	ImOpt(const ImOptName	&name,
		  Uint32			 count,
		  float				 dValue,
		  float				 min,
		  float				 max,
		  Uint32			 flags);

	ImOpt(const ImOptName	&name,
		  const char		*dValue,
		  const ImOptName	*menu,
		  Uint32			 flags);

	ImOpt(const char 		*pageName,
		  Uint32			 flags);

	ImOpt( );
	~ImOpt( );

	//
	// The following are for parsing options and can safely be ignored
	// by implementors of individual file formats.
	//
	enum   Type { tNone = 0, tToggle, tInteger, tFloat, tString, tBreak };

	inline Type				 getType( )				const;
	inline const char		*getEnglish( )			const;
	inline const char		*getTag( )				const;
	inline Uint32			 getFlags( )			const;
	inline size_t			 getOffset( )			const;
	inline Uint32			 getCountInteger( )		const;
	inline Uint32			 getCountFloat( )		const;
	inline Uint32			 getDefaultToggle( )	const;
	inline int				*getDefaultInteger( )	const;
	inline float			*getDefaultFloat( )		const;
	inline const char		*getDefaultString( )	const;
	inline int				 getMinInteger( )		const;
	inline int				 getMaxInteger( )		const;
	inline float			 getMinFloat( )			const;
	inline float			 getMaxFloat( )			const;
	inline const ImOptName	*getStringMenu( )		const;

	size_t					 getDataSize( )			const;

private:
	friend		 class ImFileMaker;
	friend		 class ImFileOptStow;

	ImOptName	 myName;
	Uint32		 myFlags;
	Type		 myType;
	size_t		 myOffset;
	ImFileMaker	*myMaker;

	union
	{
		struct
		{
			Uint32	 dValue;
		}		 myT;

		struct
		{
			int		*dValue;
			Uint32	 count;
			int		 minimum;
			int		 maximum;
		}		 myI;

		struct
		{
			float	*dValue;
			Uint32	 count;
			float	 minimum;
			float	 maximum;
		}		 myF;

		struct
		{
			const char		*dValue;
			const ImOptName	*menu;
		}		 myS;
	};
};

ImOptName::ImOptName(const char *e, const char *t)
{
	myEnglish	= e;
	myTag		= t;
}

ImOptName::ImOptName(const ImOptName &a)
{
	myEnglish	= a.myEnglish;
	myTag		= a.myTag;
}

ImOpt::Type		 ImOpt::getType( )			const { return myType; }
const char		*ImOpt::getEnglish( )		const { return myName.myEnglish; }
const char		*ImOpt::getTag( )			const { return myName.myTag; }
Uint32			 ImOpt::getFlags( )			const { return myFlags; }
size_t			 ImOpt::getOffset( )		const { return myOffset; }
Uint32			 ImOpt::getCountInteger( )	const { return myI.count; }
Uint32			 ImOpt::getCountFloat( )	const { return myF.count; }
Uint32			 ImOpt::getDefaultToggle( )	const { return myT.dValue; }
int				*ImOpt::getDefaultInteger( )const { return myI.dValue; }
float			*ImOpt::getDefaultFloat( )	const { return myF.dValue; }
const char		*ImOpt::getDefaultString( )	const { return myS.dValue; }
int				 ImOpt::getMinInteger( )	const { return myI.minimum; }
int				 ImOpt::getMaxInteger( )	const { return myI.maximum; }
float			 ImOpt::getMinFloat( )		const { return myF.minimum; }
float			 ImOpt::getMaxFloat( )		const { return myF.maximum; }
const ImOptName	*ImOpt::getStringMenu( )	const { return myS.menu; }
#endif
