//
// PROPRIETARY INFORMATION.  This software is proprietary to
// Silicon Grail Corporation, and is not to be reproduced,
// transmitted, or disclosed in any way without written permission.
//
//  Produced by:
//  Silicon Grail Corporation
//  710 N. Seward
//  Hollywood, CA
//  USA 90038
//  213-871-9100
//
// NAME: ImTrace.h
//
// COMMENTS:
//	
/* $Id: ImTrace.h,v 1.1.1.1 1999/10/06 21:12:44 build Exp $ */

#ifndef _IMTRACE_H_
#define _IMTRACE_H_

#ifdef UT_DEBUG
#define IMTRACE_ON
#endif

#include <stdio.h>
#include <stdarg.h>

#define DEFAULT_IMTRACE_LEVEL	1
#define DEFAULT_IMTRACE_FILE	stderr

#define T_READ_FILE( which, name )  ("Reading %s file `%s' ...", which, name)
#define T_WRITE_FILE( which, name ) ("Writing %s file `%s' ...", which, name)

#ifdef IMTRACE_ON
#define TRACING( debugLevel ) debugLevel >= trace.getTraceLevel()
#define TRACE( funcName, debugLevel, fmt )					\
	if ( debugLevel >= trace.getTraceLevel() ) {			\
		char s[50];											\
	    sprintf( s,"(%s)", funcName );						\
	    ImTrace::dprintf( "TRACE: %-25s -> ",s );				\
		ImTrace::dprintf fmt;									\
	    ImTrace::dprintf( "\n%-36s(line %d in file %s)\n\n",	\
						"",__LINE__,__FILE__);				\
		fflush( trace.getTraceLogFile() );					\
	}

#define DPRINT( debugLevel, fmt )							\
	if ( debugLevel >= trace.getTraceLevel() ) {			\
		ImTrace::dprintf fmt;									\
	}

#else /* !IMTRACE_ON */
#define TRACING( debugLevel )	0			
#define TRACE( funcName, debugLevel, arg )	{}
#define DPRINT( debugLevel, arg )			{}
#endif /* IMTRACE_ON */

class ImTrace {
public:
					 ImTrace(const char *module);

	Uint32			 getTraceLevel() const			{ return traceLevel; }
	static void		 setTraceLogFile(FILE *file)	{ logFile = file; }
    static FILE		*getTraceLogFile()				{ return logFile; }
    static void		 dprintf( char *fmt, ... );

private:
    Uint32			 traceLevel;
    static FILE		*logFile;
};

#endif	/* _IMTRACE_H */
