/*
 * PROPRIETARY INFORMATION.  This software is proprietary to
 * Silicon Grail Corporation, and is not to be reproduced,
 * transmitted, or disclosed in any way without written permission.
 *
 * Produced by:
 *  Kimball Thurston
 *  Silicon Grail Corporation
 *  710 N. Seward
 *  Hollywood, CA
 *  USA 90038
 *  213-871-9100
 *
 * NAME: UT_MathFunc.h
 *
 * COMMENTS: This file declares the standard lib math functions in a
 * cross platform independent way so that machines that have float (as
 * opposed to double) functions can use those to make things faster
 *
 * Be careful! This file is included by "C" code as well!
 *
 */

#ifndef _UT_MathFunc_h_
#define _UT_MathFunc_h_

#include <math.h>
#include <float.h>

#ifdef WIN32

# define UT_atoll(x)	_atoi64(x)

# define UT_fabsf(x)	(float)fabs(x)
# define UT_ffloor(x)	(float)floor(x)
# define UT_flog10(x)	(float)log10(x)
# define UT_flog(x)		(float)log(x)
# define UT_fceil(x)	(float)ceil(x)
# define UT_tanf(x)		(float)tan(x)
# define UT_acosf(x)	(float)acos(x)
# define UT_atanf(x)	(float)atan(x)
# define UT_sinhf(x)	(float)sinh(x)
# define UT_coshf(x)	(float)cosh(x)
# define UT_tanhf(x)	(float)tanh(x)
# define UT_log10f(x)	(float)log10(x)
# define UT_atan2f(x,y)	(float)atan2(x, y)
# define UT_sinf(x)		(float)sin(x)
# define UT_cosf(x)		(float)cos(x)
# define UT_fsin(x)		(float)sin(x)
# define UT_fcos(x)		(float)cos(x)
# define UT_asinf(x)	(float)asin(x)
# define UT_expf(x)		(float)exp(x)
# define UT_powf(x,y)	(float)pow(x, y)
# define UT_hypotf(x,y)	(float)hypot(x, y)
# define UT_pow(x,y)	pow(x, y)
# define UT_logf(x)		(float)log(x)
# define UT_sqrtf(x)	(float)sqrt(x)
# define UT_fsqrt(x)	(float)sqrt(x)
# define UT_ftrunc(x)	((x < 0.F) ? (float)ceil(x) : (float)floor(x))
# define UT_trunc(x)	((x < 0.F) ? ceil(x) : floor(x))

# ifdef __cplusplus
template <class T> inline T UT_rint(T a)
{ return (a >= (T)0.0) ? (T)floor((a)+0.5) : (T)ceil((a)-0.5); }
# else
#  define UT_rint(x) (x > 0.0)?((float)floor((x)+0.5)):((float)ceil((x)-0.5))
# endif

#ifndef copysign
# define copysign _copysign
#endif

#else

# define UT_atoll(x)		atoll(x)

# ifdef LINUX
#  define UT_fabsf(x)		(float)fabs(x)
#  define UT_ffloor(x)		(float)floor(x)
#  define UT_flog10(x)		(float)log10(x)
#  define UT_flog(x)		(float)log(x)
#  define UT_fceil(x)		(float)ceil(x)
#  define UT_tanf(x)		(float)tan(x)
#  define UT_acosf(x)		(float)acos(x)
#  define UT_atanf(x)		(float)atan(x)
#  define UT_sinhf(x)		(float)sinh(x)
#  define UT_coshf(x)		(float)cosh(x)
#  define UT_tanhf(x)		(float)tanh(x)
#  define UT_log10f(x)		(float)log10(x)
#  define UT_atan2f(x,y)	(float)atan2(x, y)
#  define UT_sinf(x)		(float)sin(x)
#  define UT_cosf(x)		(float)cos(x)
#  define UT_fsin(x)		(float)sin(x)
#  define UT_fcos(x)		(float)cos(x)
#  define UT_asinf(x)		(float)asin(x)
#  define UT_expf(x)		(float)exp(x)
#  define UT_powf(x,y)		(float)pow(x, y)
#  define UT_hypotf(x,y)	(float)hypot(x, y)
#  define UT_pow(x,y)		pow(x, y)
#  define UT_logf(x)		(float)log(x)
#  define UT_sqrtf(x)		(float)sqrt(x)
#  define UT_fsqrt(x)		(float)sqrt(x)
#  define UT_ftrunc(x)		((x < 0.F) ? (float)ceil(x) : (float)floor(x))
#  define UT_trunc(x)		((x < 0.F) ? ceil(x) : floor(x))
#  define UT_rint(x)		(float)rint(x)

# else

#  define UT_fabsf(x)		fabsf(x)
#  define UT_ffloor(x)		ffloor(x)
#  define UT_flog10(x)		flog10(x)
#  define UT_flog(x)		flog(x)
#  define UT_fceil(x)		fceil(x)
#  define UT_tanf(x)		tanf(x)
#  define UT_acosf(x)		acosf(x)
#  define UT_atanf(x)		atanf(x)
#  define UT_sinhf(x)		sinhf(x)
#  define UT_coshf(x)		coshf(x)
#  define UT_tanhf(x)		tanhf(x)
#  define UT_log10f(x)		log10f(x)
#  define UT_atan2f(x,y)	atan2f(x, y)
#  define UT_sinf(x)		sinf(x)
#  define UT_cosf(x)		cosf(x)
#  define UT_fsin(x)		fsin(x)
#  define UT_fcos(x)		fcos(x)
#  define UT_asinf(x)		asinf(x)
#  define UT_expf(x)		expf(x)
#  define UT_powf(x,y)		powf(x, y)
#  define UT_pow(x,y)		pow(x, y)
#  define UT_hypotf(x,y)	hypotf(x, y)
#  define UT_logf(x)		logf(x)
#  define UT_sqrtf(x)		sqrtf(x)
#  define UT_fsqrt(x)		fsqrt(x)
#  define UT_ftrunc(x)		ftrunc(x)
#  define UT_trunc(x)		trunc(x)
#  define UT_rint(x)		rint(x)

# endif
#endif

#endif
