#ifdef WIN32
#ifndef _INC_STDLIB
#include <stdlib.h>
#endif
#else
#ifndef __STDLIB_H__
#include <stdlib.h>
#endif
#endif /* WIN32 */

#if defined(linux) && !defined(LINUX)
# define LINUX
#endif

/*
 * This was adopted from some GNU stuff. It's unclear exactly why they
 * do this, but it seems like it might be a good idea, at least to avoid
 * an extra semi-colon warning message
 * NB: In their stuff, the conditionally change what is used depending
 * on the compile platform, but it seems that in most cases it will end
 * up with the do while combo, so let's leave it at that
 */
/*
 * Having the do..while(0) spits out warnings under the stupid
 * SGI compiler, so let's not use it, as it doesn't spit a warning
 * about an extraneous semi-colon
 * #define UT_ASSERT_START	do
 * #define UT_ASSERT_END while (0)
 */
#define UT_ASSERT_START
#define UT_ASSERT_END

#ifdef __cplusplus
extern "C"
{
#endif
void UT_AssertPrint(const char *msg, ...);
void UT_AssertPrintAbort(const char *msg, ...);
#ifdef __cplusplus
}
#endif
/*
 * The FATALCHECK is an assert that should ALWAYS be around - whether
 * we are building release or debug
 */
#define FATALCHECK(expr, msg) UT_ASSERT_START {\
if ( !(expr) ) {\
     UT_AssertPrintAbort( "Assertion Failure in file %s:" \
						  " line %d\n\tStatement (%s)\n\t%s\n\n", \
						  __FILE__, __LINE__, #expr, msg ); } } UT_ASSERT_END

/*
 * BAD_STMT_REACHED will most commonly be used in the default
 * of case statements in places like IM where there can only be
 * the three pixel types, and anything else is a fatal error
 */
#define BAD_STMT_REACHED(msg) UT_ASSERT_START {\
    UT_AssertPrintAbort("File %s: line %d should not have been"\
						" reached:\n\t%s\n\n",\
						__FILE__, __LINE__, msg); } UT_ASSERT_END

/*
 * UT_ASSERT is only turned on when UT_DEBUG is defined.
 * This is the assert that should be spread liberally around the code
 * as it will disappear in the release build
 */
#ifdef UT_DEBUG

#define UT_ASSERT(expr) UT_ASSERT_START {\
if ( !(expr) ) {\
    UT_AssertPrintAbort( "Assertion Failure in file %s:"\
						 " line %d\n\tStatement (%s)\n\n",\
						 __FILE__, __LINE__, #expr ); } } UT_ASSERT_END


/**************************************/


#define UT_ASSERTSTR(expr, str) UT_ASSERT_START {\
if ( !(expr) ) {\
    UT_AssertPrintAbort( "Assertion Failure in file %s:"\
						 " line %d\n\tStatement (%s)\n\tMessage = %s\n\n",\
						 __FILE__, __LINE__, #expr, str ); } } UT_ASSERT_END


/**************************************/


#define UT_ASSERTMSG(expr, msg) UT_ASSERT_START {\
if ( !(expr) ) {\
    UT_AssertPrint( "Assertion Failure in file %s:"\
						 " line %d\n\tStatement (%s)\n",\
						 __FILE__, __LINE__, #expr );\
	UT_AssertPrintAbort msg; } } UT_ASSERT_END

#else
#define UT_ASSERT(expr)
#define UT_ASSERTSTR(expr, str)
#define UT_ASSERTMSG(expr, msg)
#endif


#define ISNULL( p )		( NULL == (p) )
#define NOTNULL( p )	( NULL != (p) )


/****************************
 * Cleanup functions
 ***************************/

#define SAFE_FREE(f) if ( NULL != f ) { free(f); f = NULL; }
#define SAFE_DELETE(f) { delete f; f = NULL; }
#define SAFE_ARRDELETE(f) { delete [] f; f = NULL; }

typedef	bool	utBool;

#ifdef WIN32
typedef	signed int			Int;
typedef	signed char			Int8;
typedef	signed short		Int16;
typedef	signed long			Int32;
typedef	signed __int64		Int64;

typedef	unsigned int		Uint;
typedef	unsigned char		Uint8;
typedef	unsigned short		Uint16;
typedef	unsigned long		Uint32;
typedef	unsigned __int64	Uint64;
#elif defined(linux) && defined(__alpha__)
typedef	signed int			Int;
typedef	signed char			Int8;
typedef	signed short		Int16;
typedef	signed int			Int32;
typedef	signed long			Int64;

typedef	unsigned int		Uint;
typedef	unsigned char		Uint8;
typedef	unsigned short		Uint16;
typedef	unsigned int		Uint32;
typedef	unsigned long		Uint64;
#elif defined (linux) && defined (__i386__)
typedef	signed int			Int;
typedef	signed char			Int8;
typedef	signed short		Int16;
typedef	signed long			Int32;
typedef	signed long long	Int64;

typedef	unsigned int		Uint;
typedef	unsigned char		Uint8;
typedef	unsigned short		Uint16;
typedef	unsigned long		Uint32;
typedef	unsigned long long	Uint64;
#else
typedef	signed int			Int;
typedef	signed char			Int8;
typedef	signed short		Int16;
typedef	signed int			Int32;
typedef	signed long long	Int64;

typedef	unsigned int		Uint;
typedef	unsigned char		Uint8;
typedef	unsigned short		Uint16;
typedef	unsigned int		Uint32;
typedef	unsigned long long	Uint64;
#endif

/* Float seems to be the same on both platforms */
typedef	float			Float;
typedef	float			Float32;
typedef	double			Float64;
typedef	long double		Float128;

/*
 * Under Microsoft, if we specify dllexport, we don't need to
 * generate a def file when we create a shared object
 */
#ifndef UTDSOEXPORT
#ifdef WIN32
#define UTDSOEXPORT __declspec( dllexport )
#else
#define UTDSOEXPORT
#endif
#endif
#ifndef UTDSOIMPORT
#ifdef WIN32
#define UTDSOIMPORT __declspec( dllimport )
#else
#define UTDSOIMPORT
#endif
#endif
