/******************************************************************************
*								
*	(C) Copyright 1992 GW Hannaway & Associates, Inc.
*
*	This program is an unpublished work fully protected by the United
*	States Copyright Laws and is considered a trade secret belonging to 
*	GW Hannaway & Associates, Inc.
*
******************************************************************************/
#ifndef ERROR_H
#define ERROR_H
#include <errno.h>
#include <stdarg.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 * For Initialization, these need to be called at startup and
 * at exit, respectively, in the process that is about
 * to load the GWHA ImFile plugIns...
 */
void errorStackInit();
void errorStackCleanup();

#ifdef LINUX
#define SYSV
#endif

#if defined(sun) 
#if !defined(SYSV)
#define strerror( errornum ) sys_errlist[( errornum )]
extern char	*sys_errlist[];
#else
#include <string.h>
#endif
#endif

/*
*	If this is defined, then a file and line number of the source code
*	is appended to each error.
*/
#ifndef ERROR_LOCATION_LOGGING
/* #define ERROR_LOCATION_LOGGING */
#endif

/*
*	These defines give type of reporting for errors.
*	E_NONE		means don't report this yet
*	E_WARNING	means report this as a warning
*	E_ERROR		means report this as an error, program will not exit.
*	E_FATAL		means report this as a fatal error, the program is about
*				to exit, but gracefully
*	E_DOA		means report this as a catastropic error, the programs
*				is about to exit with no grace at all.
*/
#define E_STRENGTH_NONE		0
#define E_STRENGTH_WARNING	1
#define E_STRENGTH_ERROR	2
#define E_STRENGTH_FATAL	3
#define E_STRENGTH_DOA		4
#ifndef ERROR_LOCATION_LOGGING
#define E_NONE				E_STRENGTH_NONE,	0
#define E_WARNING( proc )	E_STRENGTH_WARNING,	proc
#define E_ERROR( proc )		E_STRENGTH_ERROR,	proc
#define E_FATAL( proc )		E_STRENGTH_FATAL,	proc
#define E_DOA( proc )		E_STRENGTH_DOA,		proc
#else /* ERROR_LOCATION_LOGGING */
#define E_NONE				__FILE__, __LINE__, E_STRENGTH_NONE,	0
#define E_WARNING( proc )	__FILE__, __LINE__, E_STRENGTH_WARNING,	proc
#define E_ERROR( proc )		__FILE__, __LINE__, E_STRENGTH_ERROR,	proc
#define E_FATAL( proc )		__FILE__, __LINE__, E_STRENGTH_FATAL,	proc
#define E_DOA( proc )		__FILE__, __LINE__, E_STRENGTH_DOA,		proc
#endif /* ERROR_LOCATION_LOGGING */

#define	E_TYPE_NONE					0x0000
#define	E_TYPE_UNKNOWN				0x00FF

/*
*	Error bases.  Add your own bases to this list.
*/
#define E_BASE_GENERAL				0x00000100
#define E_BASE_SYSTEM				0x00000200
#define E_BASE_ELM					0x00000300
#define E_BASE_SWORDFISH			0x00000500
#define E_BASE_CH					0x00000600
#define E_BASE_IM					0x00000700
#define E_BASE_TESTING				0xFFFFFE00
#define E_BASE_UNKNOWN				0xFFFFFF00

enum SystemErrorEnum {
	E_TYPE_CLOSE_FILE,
	E_TYPE_DUP,
	E_TYPE_EXEC,
	E_TYPE_FORK,
	E_TYPE_KILL,
	E_TYPE_MAKE_FIFO,
	E_TYPE_MALLOC,
	E_TYPE_OPEN_DIR,
	E_TYPE_OPEN_FILE,
	E_TYPE_READ_FILE,
	E_TYPE_SEEK_FILE,
	E_TYPE_SIGACTION,
	E_TYPE_STAT_FILE,
	E_TYPE_TIME,
	E_TYPE_WRITE_FILE
};

enum GeneralErrorEnum {
	E_TYPE_USAGE,
	E_TYPE_STRING
};

#define E_EMPTY E_TYPE_NONE, \
			0

#define E_MALLOC E_BASE_SYSTEM + E_TYPE_MALLOC, \
			"Can't allocate memory because `%s'", \
				strerror( errno )

#define E_MALLOC_ARG(object) E_BASE_SYSTEM + E_TYPE_MALLOC, \
			"Can't allocate memory for \"%s\" because `%s'", \
			object,strerror( errno )

#define E_OPEN_FILE(which,name) E_BASE_SYSTEM + E_TYPE_OPEN_FILE, \
			"Can't open %s file %s because `%s'", \
				which, name, strerror( errno )

#define E_STAT_FILE(which,name) E_BASE_SYSTEM + E_TYPE_STAT_FILE, \
			"Can't stat %s file %s because `%s'", \
				which, name, strerror( errno )

#define E_WRITE_FILE(which,name) E_BASE_SYSTEM + E_TYPE_WRITE_FILE, \
			"Can't write to %s file `%s' because `%s'", \
				which, name, strerror( errno )

#define E_READ_FILE(which,name) E_BASE_SYSTEM + E_TYPE_READ_FILE, \
			"Can't read from %s file `%s' because `%s'", \
				which, name, strerror( errno )

#define E_SEEK_FILE(which,name) E_BASE_SYSTEM + E_TYPE_SEEK_FILE, \
			"Can't seek in %s file `%s' because `%s'", \
				which, name, strerror( errno )

#define E_CLOSE_FILE(which,name) E_BASE_SYSTEM + E_TYPE_CLOSE_FILE, \
			"Can't close %s file `%s' because `%s'", \
				which, name, strerror( errno )

#define E_MAKE_FIFO(which,name) E_BASE_SYSTEM + E_TYPE_MAKE_FIFO, \
			"Can't make %s fifo `%s' because `%s'", \
				which, name, strerror( errno )

#define E_CLOSE(which) E_BASE_SYSTEM + E_TYPE_CLOSE_FILE, \
			"Can't close %s because `%s'", \
				which, strerror( errno )

#define E_DUP(which) E_BASE_SYSTEM + E_TYPE_DUP, \
			"Can't dup %s because `%s'", \
				which, strerror( errno )

#define E_OPEN_DIR(name) E_BASE_SYSTEM + E_TYPE_OPEN_DIR, \
			"Can't open directory %s because `%s'", \
				name, strerror( errno )

#define E_TIME E_BASE_SYSTEM + E_TYPE_TIME, \
			"Can't get time because `%s'", \
				strerror( errno )
    
#define E_SIGACTION E_BASE_SYSTEM + E_TYPE_SIGACTION, \
			"Can't set signal handler because `%s'", \
				strerror( errno )
    

#define E_USAGE(command,usage) E_BASE_GENERAL + E_TYPE_USAGE, \
			"Usage: %s %s", \
				command, usage

#define E_STRING(string)        E_BASE_SWORDFISH + E_TYPE_STRING, string

#define E_EXEC(name) E_BASE_SYSTEM + E_TYPE_EXEC, \
			"Can't execute %s because `%s'", \
				name, strerror( errno )

#define E_FORK E_BASE_SYSTEM + E_TYPE_FORK, \
			"Can't fork because `%s'", \
				strerror( errno )

#define E_KILL( child ) E_BASE_SYSTEM + E_TYPE_KILL, "Can't kill process # `%d' because `%s'", 	child, strerror( errno )

typedef unsigned int ErrorType;

#define ERROR_HOLDER_TEXT_LEN 128

# if defined(WIN32) || defined(LINUX)
# ifndef GWHA_ERRNO
#  define GWHA_ERRNO 2
# endif
#endif
typedef struct {
	ErrorType	type;

#ifdef GWHA_ERRNO
	int			GWHAerrno;
#else 
	int			errno;
#endif
	char		text[ERROR_HOLDER_TEXT_LEN];
} ErrorHolder;

int errorProcess(
#ifdef ERROR_LOCATION_LOGGING
			const char		*file,
			int			 line,
#endif /* ERROR_LOCATION_LOGGING */
			int			 strength,
			int			(*display)( int, const char * ),
			ErrorType	 type,
			const char	*fmt,
#if defined(sun) || defined(_ALPHA_) || defined(LINUX)
			va_list		 args
#else
			char		*args
#endif
			);

void errorV(
#ifdef ERROR_LOCATION_LOGGING
			const char		*file,
			int			line,
#endif /* ERROR_LOCATION_LOGGING */
			int			strength,
			int			(*display)( int, const char * ),
			ErrorType	type,
			const char		*fmt,
			...
			);

int	errorI(
#ifdef ERROR_LOCATION_LOGGING
			const char		*file,
			int			line,
#endif /* ERROR_LOCATION_LOGGING */
			int			strength,
			int			(*display)( int, const char * ),
			int			retval,
			ErrorType	type,
			const char		*fmt,
			...
			);

void *errorP(
#ifdef ERROR_LOCATION_LOGGING
				const char		*file,
				int			line,
#endif /* ERROR_LOCATION_LOGGING */
				int			strength,
				int			(*display)( int, const char * ),
				void		*retval,
				ErrorType	type,
				const char		*fmt,
				...
				);

int errorD(
#ifdef ERROR_LOCATION_LOGGING
				const char		*file,
				int			line,
#endif /* ERROR_LOCATION_LOGGING */
				int			strength,
				int			(*display)( int, const char * ),
				ErrorType	type,
				const char		*fmt,
				...
				);

int errorPop( void );
void errorClear( void );
int errorErrno( void );
ErrorType errorType( void );
const char *errorString( int *length );
void errorTop( void );
void errorBottom( void );
int errorUp( void );
int errorDown( void );
int errorOccurred( void );

void errorStderrInit( const char *command );

int	 errorStderr( int			 strength,
				  const char	*errorString );

void errorSyslogInit( const char *command );

int	 errorSyslog( int			 strength,
				  const char	*errorString );

ErrorHolder errorPull( void );

void errorPush( ErrorHolder hold );

typedef struct {
	ErrorType	type;
	char		*text;
#ifdef GWHA_ERRNO
	int			GWHAerrno;
#else 
	int			errno;
#endif
	int			length;
} ErrorElement;

#ifdef __cplusplus
}
#endif
#endif /* ERROR_H */
