#include <stdio.h>
#include <math.h>

/*
 * The function used to convert linear to video data is from the
 * Cinesite Digital Film Center Document:
 *
 * Grayscale Transformations of Cineon Digital Film Data
 * for Display, Conversion, and Film Recording.
 * Version 1.1, April 12, 1993.
 *
 * The original function was:
 *
 * ccir = lin > 0.018 ? 1.099 * lin ^ 0.45 - 0.099 : 4.5 * lin;
 * vid = ccir * 230 + 5;
 *
 * For the purposes of this code, the function has been inverted.
 */

#define LIN_MAX	   65535	/* 16 bin lin input */

int		white = 4095;		/* linear 90% white point */

/*
 * output table size
 */
int		entries= 256;

void
table(FILE *fp, int entries, int white)
{
	float	dout	= 1.0 / (float)(entries - 1);
	float	whitef  = (float)white / (float)LIN_MAX;
	float	lin, vid;
	int		i;

	for ( i = 0; i < entries; i++ )
	{
		vid = (float)(i - 5) / 230.F;
		lin = vid > 0.081 ? pow((vid + 0.099)/1.099, 1.0/0.45) : vid / 4.5;
		lin *= whitef;

		if ( lin < 0. )
			lin = 0.;
		if ( lin > 1.0 )
			lin = 1.;

		fprintf(fp, "%3.8f\t%3.8f\n", lin, i * dout);
	}
}

main(int argc, char **argv)
{
	FILE	*fp = stdout;

	if ( argc > 1 )
	{
		if ( (fp = fopen(argv[1], "w")) == NULL )
		{
			perror(argv[0]);
			exit(1);
		}
	}

	fprintf(fp, "# chalice display lut version 1\n");
	fprintf(fp, "# Cineon Video lookup table\n");
	fprintf(fp, "{\n" );
	fprintf(fp, "name \"Cineon Video\"\n", entries);
	fprintf(fp, "datatype_support uint16 float32\n");
	fprintf(fp, "luts 1\n");
	fprintf(fp, "entries %d\n", entries);
	fprintf(fp, "}\n" );

	fprintf(fp, "#lut 0\n");
	table(fp, entries, white);
}
