#include <stdio.h>
#include <math.h>

#define LOG_MAX		1023	/* 10 bit log input */
#define LIN_MAX	   65535	/* 16 bin lin input */

int		rmin = 140; /* The min values are empirically determined */
int		gmin = 145;
int		bmin = 90;

int		rmax = 685; /* Standard log to lin white point */
int		gmax = 685;
int		bmax = 685;

float	rgamma = 0.6;	/* Standard film gamma */
float	ggamma = 0.6;
float	bgamma = 0.6;

int		rlwhite	= 4095; /* Linear 90% white */
int		glwhite	= 4095;
int		blwhite	= 4095;

/*
 * output table size
 */
int		entries= 256;

void
table(FILE *fp, int entries, int min, int max, float gamma, int white)
{
	float	dlog	= (float)(max-min);
	float	offset	= (float)min;
	float	dout	= 1.0 / (float)(entries - 1);
	float	maxf	= (float)max;
	float	whitef	= (float)white / (float)LIN_MAX;
	float	logv, linv;
	int		i;

	for ( i = 0; i < entries; i++ )
	{
		logv = powf((i * dout), gamma) * dlog + offset;

		if (logv > (float)LOG_MAX)
			logv = (float)LOG_MAX;
		if (logv < 0.0)
			logv = 0.0;

		linv = pow( 10, (logv - maxf) * 0.002 / gamma ) * whitef;

		fprintf(fp, "%3.8f\t%3.8f\n", linv, i * dout );
	}
}

main(int argc, char **argv)
{
	FILE	*fp = stdout;

	if ( argc > 1 )
	{
		if ( (fp = fopen(argv[1], "w")) == NULL )
		{
			perror(argv[0]);
			exit(1);
		}
	}

	fprintf(fp, "# chalice display lut version 1\n");
	fprintf(fp, "# Cineview emulation lookup table\n");
	fprintf(fp, "# \t min\t max\tgamma\n");
	fprintf(fp, "# red\t%4d\t%4d\t%f\n", rmin, rmax, rgamma);
	fprintf(fp, "# green\t%4d\t%4d\t%f\n", gmin, gmax, ggamma);
	fprintf(fp, "# blue\t%4d\t%4d\t%f\n", bmin, bmax, bgamma);
	fprintf(fp, "{\n" );
	fprintf(fp, "name \"Cineview Emulation\"\n", entries);
	fprintf(fp, "datatype_support uint16 float32\n");
	fprintf(fp, "luts 3\n");
	fprintf(fp, "entries %d\n", entries);
	fprintf(fp, "}\n" );

	fprintf(fp, "#lut 0\n");
	table(fp, entries, rmin, rmax, rgamma, rlwhite);

	fprintf(fp, "#lut 1\n");
	table(fp, entries, gmin, gmax, ggamma, glwhite);

	fprintf(fp, "#lut 2\n");
	table(fp, entries, bmin, bmax, bgamma, blwhite);
}
