/*
 * User Plugin Interface
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <limits.h>
#include "cpi.h"

/*
 * Perform the solarize operation.
 * We use a template so that we can do 8bit, 16bit and float
 * images all at once
 */
template <class T>
int Solarize( T *in, T *out, float max, float mid,
				unsigned int width, unsigned int height, unsigned int channels )
{
	unsigned int numPels = width * height * channels;
	while( numPels-- )
	{
		if( (float)*in < mid )
			*out = (T)((*in)*max/mid);
		else
			*out = (T)((max-*in)*max/(max-mid));
		out++;
		in++;
	}
	return 0;
}

CPIDSOEXPORT void upiCreateParameters( void )
{
	cpiAddTab( "Solarize" );
	cpiAddFloat( "Pivot", 0.5F, 0.0F, 1.0F, P_MINMAX );
	return;
}

CPI_PluginType upiPluginType (void )
{
	return T_EFFECTS;
}

CPIDSOEXPORT int upiProcessImage( CPI_Image *result )
{
	float mid;
	cpiGetFloat( "Pivot", result->info.time, &mid );

	switch( result->info.pelType )
	{
		case P_INT8:
		{
			Solarize(	(unsigned char *)result->data, 
						(unsigned char *)result->data,
						(float)UCHAR_MAX, (float)(mid*(UCHAR_MAX)),
						result->info.sizeX, result->info.sizeY,
						result->info.channels );
			break;
		}
		case P_INT16:
		{
			Solarize(	(unsigned short *)result->data,
						(unsigned short *)result->data,
						(float)USHRT_MAX, (float)(mid*(USHRT_MAX)),
						result->info.sizeX, result->info.sizeY,
						result->info.channels );
			break;
		}
		case P_FLOAT32:
		{
			Solarize(	(float *)result->data,
						(float *)result->data,
						1.0F, mid,
						result->info.sizeX, result->info.sizeY,
						result->info.channels );
			break;
		}
		default:
			cpiError( "Unknown pixel type" );
			return 1;
	}
	return 0;
}
